#include <cmath>
#include <iostream>
#include <GL/glew.h>
#include <GL/freeglut.h>
#include <GL/freeglut_ext.h>
#include "moglclass.h"
#include "shaders.h"
#include "vertexBufferObject.h"
#include "geometryPositionColor.h"
#include "simpleCar.h"
#include "simpleRobot.h"
#include "simpleSolar.h"
#include "simpleMobile.h"
#include "simpleGeo.h"

using namespace moglclass;

#define NUMSHADERS 2
Shader shShaders[NUMSHADERS];
Program spMain;

int startTime = 0;
SimpleCar *car1;
SimpleRobot *robot1;
SimpleSolar *solar1;
SimpleMobile *mobile1;
SimpleGeo *geo1;

Cube * cube[4];

bool g_mousemove = false;
bool g_wiremode = false;
bool g_animationmode = false;
int g_transformationmode = 0;

float g_fovy = 45.0f;
float g_aspect = 1.0f;
float g_zNear = 1.0f;
float g_zFar = 100.f;

glm::vec3 g_eye(0, 0, 20.0f);
glm::vec3 g_at(0, 0, 0);
glm::vec3 g_up(0, 1, 0);

glm::mat4 World(1.0f);
glm::mat4 View(1.0f);
glm::mat4 Projection(1.0f);

float g_theta = 0.0f;

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

// set geometry
void setData()
{
	car1 = new SimpleCar();
	robot1 = new SimpleRobot();
	solar1 = new SimpleSolar();
	mobile1 = new SimpleMobile();
	geo1 = new SimpleGeo();

	cube[0] = new Cube();
	cube[0]->setColor(glm::vec3(1.0f, 0.0f, 1.0f));
	cube[1] = new Cube();
	cube[1]->setColor(glm::vec3(1.0f, 0.0f, 0.0f));
	cube[2] = new Cube();
	cube[2]->setColor(glm::vec3(0.0f, 1.0f, 0.0f));
	cube[3] = new Cube();
	cube[3]->setColor(glm::vec3(0.0f, 0.0f, 1.0f));
}

void init( void )
{
    // Load shaders and use the resulting shader program
	spMain.createProgram();
	shShaders[0].loadShader("simple4.vs", GL_VERTEX_SHADER);
	shShaders[1].loadShader("simple4.fs", GL_FRAGMENT_SHADER);
	spMain.attachShaderToProgram(&shShaders[0]);
	spMain.attachShaderToProgram(&shShaders[1]);
	spMain.linkProgram();
	spMain.useProgram();

	setData();
	
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glClearColor( 1.0, 1.0, 1.0, 1.0 ); // white background
}

void display( void )
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); // clear the window

	// set drawmode
	if (g_wiremode) 
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);  // draw wireframe 
	else
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	// draw filled 

	// MVP matrix
	Projection = glm::perspective(g_fovy, g_aspect, g_zNear, g_zFar);
	View = glm::lookAt(g_eye, g_at, g_up);
	spMain.useProgram();
	spMain.setUniform("gProjection", Projection);
	spMain.setUniform("gView", View);	

	// p' = M3 * M2 * M1 * p (OpenGL uses Column-Major Order)
	glm::mat4 Tx = glm::translate(glm::mat4(1.0f), glm::vec3(3.0f, 0.0f, 0.0f)); // RHS x+ right
	glm::mat4 Rz = glm::rotate(glm::mat4(1.0f), 45.0f, glm::vec3(0.0f, 0.0f, 1.0f)); // RHS z+ (X->Y rotation)
	glm::mat4 S = glm::scale(glm::mat4(1.0f), glm::vec3(0.5f, 0.7f, 1.0f)); // RHS

	World = glm::mat4(1.0f);
	switch (g_transformationmode)
	{
		case 0:
			World = glm::mat4(1.0f);
			spMain.setUniform("gModel", World);
			cube[0]->draw();
			break;
		case 1:
			// p'= R T p (red) => translate, and then rotate
			World = Rz * Tx; // Translate X, and then Rotate Z
			spMain.setUniform("gModel", World);
			cube[1]->draw();
			break;
		case 2:
			// p'= T R p (green) => rotate, and then translate
			World = Tx * Rz; // Rotate Z, and then Translate X
			spMain.setUniform("gModel", World);
			cube[2]->draw();
			break;
		case 3:
			// p'= T R S p (blue) => scale, and then rotate, and then translate 
			World = Tx * Rz * S; // Scale XY, and then Rotate Z, and then Translate X
			spMain.setUniform("gModel", World);
			cube[3]->draw();
			break;
		case 4:
			car1->draw(&spMain, Projection, View, World);
			break;
		case 5:
			robot1->draw(&spMain, Projection, View, World);
			break;
		case 6:
			solar1->draw(&spMain, Projection, View, World);
			break;
		case 7:
			mobile1->draw(&spMain, Projection, View, World);
			break;
		case 8:
			geo1->draw(&spMain, Projection, View, World);
			break;
	}
	glBindVertexArray(0);
	glDisableVertexAttribArray(0);

	glutSwapBuffers();
}

void update()
{
	static int currentTime, deltaTime, prevTime = 0; 
	static int elapsedTime = 0;
	currentTime = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = currentTime - prevTime;
	elapsedTime = currentTime - startTime;

	car1->update((float)deltaTime);
	robot1->update((float)deltaTime);
	solar1->update((float)deltaTime);
	mobile1->update((float)deltaTime);
	geo1->update((float)deltaTime);

	elapsedTime = currentTime - startTime;
	prevTime = currentTime;

	glutPostRedisplay();
}

void keyboard( unsigned char key, int x, int y )
{
	switch ( key ) {
		case 27:
			exit( EXIT_SUCCESS );
			break;
		case 'w':
			g_wiremode = !g_wiremode;
			break;
		case 'a':
			g_animationmode = !g_animationmode;
			break;
		case 't':
			g_transformationmode = (g_transformationmode + 1) % 4;
			printf("g_transformationmode=%d\n", g_transformationmode);
			break;
		case 'c':
			g_transformationmode = 4;
			if (!car1->isActive()) car1->activate();
			else car1->deactivate();
			break;
		case 'r':
			g_transformationmode = 5;
			if (!robot1->isActive()) robot1->activate();
			else robot1->deactivate();
			break;
		case 's':
			g_transformationmode = 6;
			if (!solar1->isActive()) solar1->activate();
			else solar1->deactivate();
			break;
		case 'm':
			g_transformationmode = 7;
			if (!mobile1->isActive()) mobile1->activate();
			else mobile1->deactivate();
			break;
		case 'g':
			g_transformationmode = 8;
			if (!geo1->isActive()) geo1->activate();
			else geo1->deactivate();
			break;
	}
	glutPostRedisplay();
}

void specialkey(int key, int x, int y )
{
	switch (key)  
	{    
		case GLUT_KEY_LEFT:
			robot1->setTheta(g_theta-=10.0f);
			break;  
		case GLUT_KEY_RIGHT:    
			robot1->setTheta(g_theta+=10.0f);
			break;  
	}
	glutPostRedisplay();
}

void mouse(int button, int state, int mx, int my)
{
	if (button==GLUT_RIGHT_BUTTON && state==GLUT_DOWN)
		exit(0);
	if (button==GLUT_LEFT_BUTTON && state==GLUT_DOWN)
		g_mousemove = true;
	else if (button==GLUT_LEFT_BUTTON && state==GLUT_UP)
		g_mousemove = false;
}

void motion(int mx, int my)
{
	int w = glutGet(GLUT_WINDOW_WIDTH);
	int h = glutGet(GLUT_WINDOW_HEIGHT);

	// 0~600(x+ right) x 0~600(y+ down) => -5~5(x+ right) x -5~5(y+ up)
	float x = (float)  10.0f * (mx - w*0.5f) / w;
	float y = (float) 10.0f * (h*0.5f - my) / h;

	if (g_mousemove)
	{
		//printf("mx=%d my=%d x=%f y=%f\n", mx, my, x, y);
		//geoList[g_geometrymode]->setPosition(glm::vec3(x, y, 0));
	}

	glutPostRedisplay();
}

void reshape(int w, int h)
{
	g_aspect = (float) (w/h);

    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glutPostRedisplay();
}

void cleanup()
{		 
	spMain.deleteProgram();
}

int main( int argc, char **argv )
{ 
	glutInit(&argc, argv);
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA  | GLUT_DEPTH);
	glutInitWindowSize( 600, 600 );
	glutCreateWindow( "Geometry Primitives Transformation" );

	glewInit();
	init();
	glutDisplayFunc( display );
	glutKeyboardFunc( keyboard );
	glutSpecialFunc( specialkey );
	glutMouseFunc( mouse ); 
	glutMotionFunc( motion ); 
	glutIdleFunc( update );
	glutReshapeFunc( reshape );
	glutMainLoop();

	cleanup();
	return 0;
}