//////////////////////////////////////////////////////////////////////
// car.cpp: implementation for the Car class (Hierarchical Transformation).
// -Kyoung Shin Park (2013 Spring)
//////////////////////////////////////////////////////////////////////
#include "simpleCar.h"

namespace moglclass
{
	
SimpleCar::SimpleCar()
{
	angle = 0.0f; // 0~360
	direction = 1.0f; // +/-1
	position = glm::vec3(-5.0f, 0.0f, 0.0f);

	init();
}

void SimpleCar::init()
{
	body = Parallelepiped(glm::vec3(-1.0f, 0.0f, -1.0f), glm::vec3(2.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 2.0f), glm::vec3(0.0f, 1.0f, 0.0f));
	body.setColor(glm::vec3(0.0f, 1.0f, 0.0f));
	wheel[0] = Torus(glm::vec3(0.0f, 0.0f, .0f), 0.3f, 0.1f, 32, 16);
	wheel[0].setColor(glm::vec3(1.0f, 0.0f, 0.0f));
	wheel[1] = Torus(glm::vec3(0.0f, 0.0f, .0f), 0.3f, 0.1f, 32, 16);
	wheel[1].setColor(glm::vec3(0.0f, 0.0f, 1.0f));

	bodyTransform = glm::translate(glm::mat4(1.0f), position); // RHS x+ right
	wheelTransform[0] = glm::translate(glm::mat4(1.0f), glm::vec3(-0.5f, 0.0f, -0.5f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f));
	wheelTransform[1] = glm::translate(glm::mat4(1.0f), glm::vec3(-0.5f, 0.0f, 0.5f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f));
	wheelTransform[2] = glm::translate(glm::mat4(1.0f), glm::vec3(0.5f, 0.0f, -0.5f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f));
	wheelTransform[3] = glm::translate(glm::mat4(1.0f), glm::vec3(0.5f, 0.0f, 0.5f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f));
}

void SimpleCar::draw(Program* p, glm::mat4& projection, glm::mat4& view, glm::mat4& model)
{
	p->useProgram();
	p->setUniform("gProjection", projection);
	p->setUniform("gView", view);

	glm::mat4 carMatrix = model * bodyTransform;
	p->setUniform("gModel", carMatrix);
	body.draw();

	glm::mat4 wheelMatrix = model * bodyTransform * wheelTransform[0];
	p->setUniform("gModel", wheelMatrix);
	wheel[0].draw();

	wheelMatrix = model * bodyTransform * wheelTransform[1];
	p->setUniform("gModel", wheelMatrix);
	wheel[0].draw();

	wheelMatrix = model * bodyTransform * wheelTransform[2];
	p->setUniform("gModel", wheelMatrix);
	wheel[1].draw();

	wheelMatrix = model * bodyTransform * wheelTransform[3];
	p->setUniform("gModel", wheelMatrix);
	wheel[1].draw();
}

bool SimpleCar::update(float deltaTime)
{
	angle = angle - 180.0f * (float) (deltaTime) * 0.0001 * direction;
	position[0] = position[0] + (float) (deltaTime) * 0.0001 * direction;

	//std::cout << "position[0]=" << position[0] << std::endl;
	if (position[0] * direction > 5)
		direction = -direction;

	bodyTransform = glm::translate(glm::mat4(1.0f), position); // RHS x+ right
	wheelTransform[0] = glm::translate(glm::mat4(1.0f), glm::vec3(-0.5f, 0.0f, -0.5f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f)); 
	wheelTransform[1] = glm::translate(glm::mat4(1.0f), glm::vec3(-0.5f, 0.0f,  0.5f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f)); 
	wheelTransform[2] = glm::translate(glm::mat4(1.0f), glm::vec3( 0.5f, 0.0f, -0.5f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f)); 
	wheelTransform[3] = glm::translate(glm::mat4(1.0f), glm::vec3( 0.5f, 0.0f,  0.5f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f)); 

	return true;
}

} // moglclass namespace