#include "simpleGeo.h"

namespace moglclass
{
SimpleGeo::SimpleGeo()
{
	theta = 0.0f; // 0~360
	phi = 0.0f; // 0~360
	psi = 0.0f; // 0~30
	direction = 1.0f; // +/-1

	active = false;
	init();
}

SimpleGeo::~SimpleGeo()
{
	std::cout << "~SimpleGeo" << std::endl;
}

void SimpleGeo::init()
{
	geo[0] = new Cylinder(glm::vec3(0, 0, 0), 2.5f, 2.0f, 5, 16); // body
	geo[0]->setColor(glm::vec3(1, 0, 0));
	geo[1] = new Sphere(glm::vec3(0, 0, 0), 1.5f, 16, 16); // head
	geo[1]->setColor(glm::vec3(0, 1, 0));
	geo[2] = new Cylinder(glm::vec3(0, 0, 0), 1, 1, 3, 8); // arm
	geo[2]->setColor(glm::vec3(0, 0, 1));
	geo[3] = new Parallelepiped(glm::vec3(-0.5f, 0.0f, -0.5f), glm::vec3(1, 0, 0), glm::vec3(0, 0, 1), glm::vec3(0, -3, 0));
	geo[3]->setColor(glm::vec3(1, 0, 1));

	setTransformation();
}

void SimpleGeo::draw(Program* p, glm::mat4& projection, glm::mat4& view, glm::mat4& model)
{
	p->useProgram();
	p->setUniform("gProjection", projection);
	p->setUniform("gView", view);

	// body
	glm::mat4 mMatrix = model * transform[0];
	p->setUniform("gModel", mMatrix);
	geo[0]->draw();

	// head
	mMatrix = model * transform[0] * transform[1];
	p->setUniform("gModel", mMatrix);
	geo[1]->draw();

	// upperArm Left
	mMatrix = model * transform[0] * transform[2];
	p->setUniform("gModel", mMatrix);
	geo[2]->draw();

	// upperArm Right
	mMatrix = model * transform[0] * transform[3];
	p->setUniform("gModel", mMatrix);
	geo[2]->draw();

	// lowerArm Left
	mMatrix = model * transform[0] * transform[2] * transform[4];
	p->setUniform("gModel", mMatrix);
	geo[3]->draw();

	// lowerArm Right
	mMatrix = model * transform[0] * transform[3] * transform[4];
	p->setUniform("gModel", mMatrix);
	geo[3]->draw();
}

void SimpleGeo::setTransformation()
{
	transform[0] = glm::rotate(glm::mat4(1.0f), theta, glm::vec3(0.0f, 1.0f, 0.0f));
	transform[1] = glm::translate(glm::mat4(1.0f), glm::vec3(0, 6.5f, 0));
	transform[2] = glm::translate(glm::mat4(1.0f), glm::vec3(3.0f, 1, 0)) * glm::rotate(glm::mat4(1.0f), phi, glm::vec3(0.0f, 1.0f, 0.0f));
	transform[3] = glm::translate(glm::mat4(1.0f), glm::vec3(-3.0f, 1, 0)) * glm::rotate(glm::mat4(1.0f), phi, glm::vec3(0.0f, 1.0f, 0.0f));
	transform[4] = glm::translate(glm::mat4(1.0f), glm::vec3(0, 0.5, 0)) * glm::scale(glm::mat4(1.0f), glm::vec3(0.5f, 1.0f, 0.5f)) * glm::rotate(glm::mat4(1.0f), psi, glm::vec3(0.0f, 0.0f, 1.0f));
}

bool SimpleGeo::update(float deltaTime)
{
	if (active)
	{
		theta += 0.001f * deltaTime;
		phi += 0.002f * deltaTime;
		psi += 0.003f * deltaTime * direction;
		if (psi * direction > 1.0)
			direction = -direction;
	}

	setTransformation();
	return true;
}
}
